@@ -1,4 +1,5 @@
 from patience.action import Action
+from patience.structures import ActionException
 
 
 class Unittests(Action):
@@ -6,9 +7,11 @@ class Unittests(Action):
     def __init__(self): 
         Action.__init__(self, parallel=False, any_order=False)
 
-    def single_action_starting(self, resource): pass
+    def single_action_starting(self, resource): 
+        pass
 
-    def single_action_result_display(self, resource, result): pass
+    def single_action_result_display(self, resource, result): 
+        pass
     
     def single_action(self, r):
         if r.is_downloaded():
